package test;

import bfv.util.TextAreaOutputStream;

import javax.bluetooth.*;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: Alistair
 * Date: 28/04/2011
 * Time: 8:36:41 PM
 * To change this template use File | Settings | File Templates.
 */
public class BFVDeviceList extends JList implements ActionListener, DiscoveryListener {

    private DiscoveryAgent agent;
    private String connectionURL;
    private ArrayList devices;
    private DefaultListModel model;
    private StreamConnection connection;
    private TextAreaOutputStream stream;
    private BFVTestGUI gui;
    private JFrame frame;

    public BFVDeviceList(DiscoveryAgent agent, TextAreaOutputStream stream, BFVTestGUI gui) {
        frame = new JFrame("Bluetooth Devices");
        this.stream = stream;
        this.agent =  agent;
        this.gui = gui;
        JPanel top = new JPanel();


        this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        this.setLayoutOrientation(JList.HORIZONTAL_WRAP);
        this.setVisibleRowCount(-1);

        JScrollPane scroll = new JScrollPane(this);
        scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scroll.setPreferredSize(new Dimension(250, 80));
        top.add(scroll);

        JPanel bottom = new JPanel();

        JButton refresh = new JButton("Inquire");
        refresh.setActionCommand("refresh");
        refresh.addActionListener(this);
        bottom.add(refresh);

        JButton connect = new JButton("Connect");
        connect.setActionCommand("connect");
        connect.addActionListener(this);
        bottom.add(connect);


        JButton cancel = new JButton("Cancel");
        cancel.setActionCommand("cancel");
        cancel.addActionListener(this);
        bottom.add(cancel);

        model = new DefaultListModel();

        devices = new ArrayList();

        RemoteDevice[] cached = agent.retrieveDevices(DiscoveryAgent.CACHED);

        
        for (int i = 0; i < cached.length; i++) {
            RemoteDevice remoteDevice = cached[i];
            this.stream.println(remoteDevice);
            devices.add(remoteDevice);
        }
        updateList();

        this.setModel(model);


        frame.getContentPane().add(top, BorderLayout.CENTER);
        frame.getContentPane().add(bottom, BorderLayout.PAGE_END);
        frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);





    }

    public JFrame getFrame() {
        return frame;
    }

    public void updateList(){
        model.clear();


        for (int i = 0; i < devices.size(); i++) {
            RemoteDevice remoteDevice = (RemoteDevice) devices.get(i);
            try {

                model.addElement(remoteDevice.getFriendlyName(false) + "(" + remoteDevice.getBluetoothAddress() + ")");
            } catch (IOException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }

        frame.pack();


    }

    public void actionPerformed(ActionEvent e) {
        if(e.getActionCommand().equals("refresh")){
            try {
                devices = new ArrayList();
                updateList();
                agent.startInquiry(DiscoveryAgent.GIAC, this);

                } catch (BluetoothStateException e1) {
                e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }


        }
        else if(e.getActionCommand().equals("connect")){
            RemoteDevice device = (RemoteDevice)devices.get(this.getSelectedIndex());
            UUID[] uuidSet = new UUID[1];
		    uuidSet[0]=new UUID("1101",true);
            try {
                agent.searchServices(null, uuidSet, device, this);
            } catch (BluetoothStateException e1) {
                e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }


        }
        else if(e.getActionCommand().equals("cancel")){
           frame.setVisible(false);

        }
    }

    public void deviceDiscovered(RemoteDevice remoteDevice, DeviceClass deviceClass) {

        devices.add(remoteDevice);
        updateList();



    }

    public void servicesDiscovered(int i, ServiceRecord[] serviceRecords) {
        if(serviceRecords!=null && serviceRecords.length>0){
			connectionURL=serviceRecords[0].getConnectionURL(0,false);
		}

    }

    public void serviceSearchCompleted(int i, int i1) {
        if(connectionURL==null){
			stream.println("Device does not support SPP");
		}
		else{
			stream.println("Device supports SPP");
            stream.println(connectionURL);
            //connect to the server and send a line of text
            try {
                connection=(StreamConnection) Connector.open(connectionURL);
                frame.setVisible(false);
                gui.deviceConnected(connection);


            } catch (IOException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }

        }
    }

    public void inquiryCompleted(int i) {
          updateList();

    }

    public StreamConnection getConnection() {
        return connection;
    }
}
